<?php

require_once('Slim/Slim.php');
require_once('includes/_dbconfig.php');
require_once('includes/AfricasTalkingGateway.php');
require_once('includes/PasswordHash.php');


\Slim\Slim::registerAutoloader();

$app = new \Slim\Slim();

//Common Actions
$app->delete('/phonefinder/delete/', 'deleteSelected');
$app->post('/phonefinder/compareimages/', 'getSmartPhonesImages');
$app->post('/phonefinder/compare/', 'getSmartPhones');

$app->post('/openday/apps/', 'sendApps');
$app->post('/openday/user/', 'addOpenUser');
$app->get('/openday/users/', 'getOpenUsers');
$app->post('/openday/sms/', 'sendAnSMS');

//Smart Phones Images
$app->get('/phonefinder/smart_phone/images/:id', 'getSmartPhoneImages'); 
$app->get('/phonefinder/smart_phone/images/', 'getAllSmartPhoneImages');

//Smart Phones
$app->get('/phonefinder/smart_phone/', 'getAllSmartPhones');
$app->get('/phonefinder/smart_phone/:id', 'getSmartPhone');
$app->get('/phonefinder/smart_phone/device_type/:id', 'getSmartPhoneByDeviceCategory');

$app->get('/phonefinder/smart_phone_detail/:id', 'getSmartPhoneDetails');

$app->post('/phonefinder/smart_phone/searchbyprice/', 'searchSmartPhonesByPrice');
$app->post('/phonefinder/smart_phone/searchbyos/', 'searchSmartPhonesByOS');
$app->post('/phonefinder/smart_phone/searchbymanufacturer/', 'searchSmartPhonesByManufacturer');
$app->post('/phonefinder/smart_phone/searchbykeyword/', 'searchSmartPhonesByKeyword');
$app->post('/phonefinder/smart_phone/searchbydevicetype/', 'searchSmartPhonesByDeviceType');


//Accessory

$app->get('/phonefinder/accessory/', 'getAllAccessories');
$app->get('/phonefinder/accessory/:id', 'getAccessory');

$app->run();




/**
 * Function Name: sendApps
 *
 * 
 * Sends a single SMSs with apps links to a phone number
 *
 * @param (text) ($sms_message) this is the SMS text to be sent
 * @param (number) ($recipient_phone) this is the recipient mobile phone number
 * @return (json) (Phone Number: Status: Cost:) the SMS gateway returns this
 *
 */
function sendApps() {

	$request = \Slim\Slim::getInstance()->request();
	$sms = json_decode($request->getBody());

	
	$recipient_phone =  preg_replace('/\s+/', '', $sms->user_phone);//remove spaces
	$recipient_name = $sms->user_name;
	$apps_urls = $sms->apps_urls;
	$category_heading = $sms->category_heading;
	$recipient_id = $sms->openuser_id;
	$recipient_apps = $sms->apps_names;

	$sms_message = 'Hi '.$recipient_name.', these are the '.$category_heading.' you requested for: '.$apps_urls.' -- '.SHOP_NAME;


	// Specify your login credentials
	$username = "izone";
	$api_Key = "dbaa3246a6ef85e54aaf918bd930589cd8ca17a07530721cc6046dd44634f220";

	// Create a new instance of AfricasTalkingGateway 
	$gateway = new AfricaStalkingGateway($username, $api_Key);
	
	// Thats it, hit send and we'll take care of the rest
	$results = $gateway->sendMessage($recipient_phone, $sms_message);
	
	$api_results = array();
	
	if ( count($results) ) {
		// These are the results if the request is well formed
		foreach($results as $result) {
			$num = " Number: " .$result->number;
			$stat = " Status: " .$result->status;
			$cost = " Cost: " .$result->cost;
			$msgid = " Message Id: " .$result->messageId.'<br/>';
			$api_results[] = $num.$stat.$cost.$msgid;						
		}
		
		$responce_string = implode(',', json_decode(json_encode($api_results),true));

		addOpenUserApplications($recipient_id, $recipient_name, $recipient_phone, $recipient_apps);

		$message = 'Your request has been processed, you will receive an SMS shortly.';		
		echo '{"message":'. json_encode($message) .'}'; //echo responce
		
		
	} else{
		// We only get here if we cannot process
		// (usually due to wrong username/apike
		echo '{"error":'. json_encode('Oops, No SMS was sent.<br/> Error Message: '.$gateway->getErrorMessage()) .'}';
	}// DONE!!!
	

}






/**
 * Function Name: addUser
 *
 * Adds a new admin user 
 *
 * @param (array) ([user details]) the admin user details
 * @return (number) ($rows) the number of rows added
 *
 */
function addOpenUser() {
	
	$request = \Slim\Slim::getInstance()->request();
	$user = json_decode($request->getBody());

	$sql = "INSERT INTO open_users (user_name, phone_number, security_token) VALUES (:user_name, :phone_number, :security_token)";
	    
	    try {		    
		    
			$flag = 'open_user';	
			
		    if(exists($user->user_phone, $flag)){
				$message = 'Welcome back, '.$user->user_name.' enjoy the experience.';
				$user_details = getOpenUser($user->user_phone);
				echo '{"welcomeback":'. json_encode($message) .', "userid": '.json_encode($user_details->id).'}'; //echo responce
		    }else{
		    	
		    	$db = getConnectionTwo();
				$stmt = $db->prepare($sql);	

				$name = cleanField($user->user_name);			
				
				$hasher = new PasswordHash(8, FALSE);				
				$token = $hasher->HashPassword($user->user_phone);
				
				$stmt->bindParam("user_name", $name);
				$stmt->bindParam("phone_number", $user->user_phone);				
				$stmt->bindParam("security_token", $token);			
				
				$rows = $stmt->execute();
				$user_id = $db->lastInsertId();

				$db = null;

				if($rows > 0){				   
				    $message = 'Your request has been processed. Thank you';		
					echo '{"message":'. json_encode($message) .' , "userid": '.json_encode($user_id).'}'; //echo responce
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }		   
	
}







/**
 * Function Name: getOpenUsers
 *
 * Retrieves all open users details 
 * 
 * @return (object) ($users) the actual users details
 */
function getOpenUsers() {

	$sql ="SELECT id, user_name, phone_number FROM open_users";		
		
	try {
		$db = getConnectionTwo();
		$stmt = $db->query($sql);		  
		$users = $stmt->fetchAll(PDO::FETCH_OBJ); 
		$db = null;
		echo '{"users":'. json_encode($users) .'}'; 
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}




/**
 * Function Name: getOpenUser
 *
 * Retrieves open user details by phonenumber
 * 
 * @return (object) ($user_details) the actual users details
 */
function getOpenUser($phone_number) {

	$sql ="SELECT id, user_name, phone_number FROM open_users WHERE phone_number = :phone";		
		
	try {
		$db = getConnectionTwo();
		$stmt = $db->prepare($sql);
		$stmt->bindParam("phone", $phone_number);
		$stmt->execute();		  
		$user_details = $stmt->fetchObject();
		$db = null;
		//echo '{"openuser":'. json_encode($user_details) .'}'; 
		return $user_details;
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}






/**
 * Function Name: sendAnSMS
 *
 * Sends a single SMSs to a single phone number
 *
 * @param (text) ($sms_message) this is the SMS text to be sent
 * @param (number) ($recipient_phone) this is the recipient mobile phone number
 * @return (json) (Phone Number: Status: Cost:) the SMS gateway returns this
 *
 */
function sendAnSMS() {

	$request = \Slim\Slim::getInstance()->request();
	$sms = json_decode($request->getBody());

	
	$recipient_phone =  preg_replace('/\s+/', '', $sms->user_number);//remove spaces
	$sms_message = $sms->message;		
	

	// Specify your login credentials
	$username = "izone";
	$api_Key = "dbaa3246a6ef85e54aaf918bd930589cd8ca17a07530721cc6046dd44634f220";

	// Create a new instance of AfricasTalkingGateway 
	$gateway = new AfricaStalkingGateway($username, $api_Key);
	
	// Thats it, hit send and we'll take care of the rest
	$results = $gateway->sendMessage($recipient_phone, $sms_message);
	
	$api_results = array();
	
	if ( count($results) ) {
		// These are the results if the request is well formed
		foreach($results as $result) {
			$num = " Number: " .$result->number;
			$stat = " Status: " .$result->status;
			$cost = " Cost: " .$result->cost;
			$msgid = " Message Id: " .$result->messageId.'<br/>';
			$api_results[] = $num.$stat.$cost.$msgid;						
		}
		
		$responce_string = implode(',', json_decode(json_encode($api_results),true));
		
		echo '{"message":'. json_encode($responce_string) .'}'; //echo responce
		
	} else{
		// We only get here if we cannot process
		// (usually due to wrong username/apike
		echo '{"error":'. json_encode('Oops, No Message were sent.<br/> Error Message: '.$gateway->getErrorMessage()) .'}';
	}// DONE!!!
	
	

}




/**
 * Function Name: addOpenUserApplications
 *
 * Adds all applications that have been sent via SMS to an open user
 *
 * @param (integer) ($id) the id of the open user
 * @param (text) ($name) the name of the open user
 * @param (text) ($phone) the phone number of the open user
 * @param (text) ($apps) the applications names sent via SMS
 * 
 *
 */
function addOpenUserApplications($id, $name, $phone, $apps) {
	

	$sql = "INSERT INTO open_user_apps (open_user_id, open_user_name, open_user_phone, open_user_applications) VALUES (:open_user_id, :open_user_name, :open_user_phone, :open_user_applications)";
	    
	    try {		    
		    	
		    	$db = getConnectionTwo();
				$stmt = $db->prepare($sql);	

				$openuser_name = cleanField($name);
				$openuser_phone = cleanField($phone);
				$openuser_apps = cleanField($apps);			
				
				$stmt->bindParam("open_user_id", $id);
				$stmt->bindParam("open_user_name", $openuser_name);
				$stmt->bindParam("open_user_phone", $openuser_phone);				
				$stmt->bindParam("open_user_applications", $openuser_apps);			
				
				$rows = $stmt->execute();
				
				$db = null;   
		   
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }		   
	
}







/**
 * Function Name: getAccessory
 *
 * Returns details of an accessory under a certain id
 *
 * @param (number) (id) this is the id of the accessory
 * @return (object) ($user) the actual accessory details
 */
function getAccessory($id) {	
	
	$sql ="SELECT * FROM smart_phone_accessories WHERE acc_id = :id ";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$accessory = $stmt->fetchObject();  
		$db = null;
		echo '{"accessory":'. json_encode($accessory) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}








/**
 * Function Name: getAllAccessories
 *
 * Returns all accessories from teh database
 * 
 * @return (object) ($accessories) the actual accessories details
 */
function getAllAccessories() {	
	
	$sql ="SELECT * FROM smart_phone_accessories ORDER BY acc_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);		
		$stmt->execute();
		$accessories = $stmt->fetchAll(PDO::FETCH_OBJ);  
		$db = null;
		echo '{"accessories":'. json_encode($accessories) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}





/**
 * Function Name: deleteAccessory
 *
 * Deletes a single accessory
 *
 * @param (number) (id) this is the id of the accessory to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteAccessory($id) {	
	
	$sql = "DELETE FROM smart_phone_accessories WHERE acc_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete an accessory!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){				
			    echo '{"message":'. json_encode($rows .' Accessory Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}





/**
 * Function Name: addAccessory
 *
 * Adds a new application 
 *
 * @param (array) ([application details]) the application details
 * @return (number) ($rows) the number of rows added
 *
 */
function addAccessory() {
	
	$request = \Slim\Slim::getInstance()->request();	
	$body = $request->getBody();
	$phone_accessory = json_decode($body);

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add a new accessory!"}';
	}else{
			
		$sql = "INSERT INTO smart_phone_accessories (accessory_name, accessory_image_name, accessory_description, accessory_price) VALUES (:accessory_name, :accessory_image_name, :accessory_description, :accessory_price)";
	    
	    $accessory_name = cleanField($phone_accessory->accessory_name);
	    $accessory_image_name = cleanField($phone_accessory->accessory_image_name);
	    $accessory_description = cleanField($phone_accessory->accessory_description);
	    $accessory_price = cleanField($phone_accessory->accessory_price);
	    
	    try {
		    
		    
			$flag = 'accessory';		    
		    if(exists($accessory_name, $flag)){
				echo '{"error":'. json_encode('Accessory Name is not Available, another accessory has a similar name') .'}';
		    }else{
		    	
		    	$db = getConnection();
				$stmt = $db->prepare($sql);
				
				$stmt->bindParam("accessory_name", $accessory_name);
				$stmt->bindParam("accessory_image_name", $accessory_image_name);
				$stmt->bindParam("accessory_description", $accessory_description);  
				$stmt->bindParam("accessory_price", $accessory_price);				
				
				$rows = $stmt->execute();

				$db = null;
				
				if($rows > 0){
				    echo '{"message":'. json_encode($rows .' New Accessory Added').'}';			
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}







/**
 * Function Name: updateAccessory
 *
 * Updates a single application 
 *
 * @param (array) ([application details]) the application details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateAccessory($id) {

	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$phone_accessory = json_decode($body);
	

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to edit an accessory!"}';
	}else{			
		
		$sql = "UPDATE smart_phone_accessories SET accessory_name =:accessory_name, accessory_image_name =:accessory_image_name, accessory_description =:accessory_description, accessory_price =:accessory_price WHERE acc_id =:id";

	    $accessory_name = cleanField($phone_accessory->accessory_name);
	    $accessory_image_name = cleanField($phone_accessory->accessory_image_name);
	    $accessory_description = cleanField($phone_accessory->accessory_description);
	    $accessory_price = cleanField($phone_accessory->accessory_price);
	    
	    try {
		    
		    
			$flag = 'accessory';		    
		    if(updateExists($accessory_name, $id, $flag)){
				echo '{"error":'. json_encode('Accessory Name is NOT available, another accessory has a similar name') .'}';
		    }else{
		    	
		    	$db = getConnection();
				$stmt = $db->prepare($sql);	

				$stmt->bindParam("id", $id);				
				$stmt->bindParam("accessory_name", $accessory_name);
				$stmt->bindParam("accessory_image_name", $accessory_image_name);
				$stmt->bindParam("accessory_description", $accessory_description);  
				$stmt->bindParam("accessory_price", $accessory_price);
				
				$rows = $stmt->execute();
				
				$db = null;
				
				if($rows > 0){				    
				    echo '{"message":'. json_encode('Accessory details Updated!') .'}';			
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: getSmartPhones
 *
 * Returns all phones on search
 *
 * @param (number) (ids) this is id array of the resource to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function getSmartPhones() {	

	$request = \Slim\Slim::getInstance()->request();
	$phone_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$id_list = $phone_details['ids'];
	$count = count($id_list);	

	try {
		
		$object_array = array();

		for($i=0;$i<$count;$i++){

			$id = $id_list[$i];
			$object_array[] = getSmartPhoneInner($id);

		}
		
		echo '{"phones":'. json_encode($object_array) .'}'; 
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}








/**
 * Function Name: searchSmartPhonesByPrice
 *
 * Returns all phones on search by price
 *
 * @param (array) (price_list) this is an array of the min and max price range
 * @return (object) ($smartphones) an array of the phones that fall in that price range
 */
function searchSmartPhonesByPrice() {	

	$request = \Slim\Slim::getInstance()->request();
	$price_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$price_list = $price_details['prices'];
	$min_price = $price_list[0];
	$max_price = $price_list[1];	

	$sql ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones WHERE smart_current_price BETWEEN :min_price AND :max_price ORDER BY smart_current_price";
	

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->bindParam(":min_price",$min_price);
		$stmt->bindParam(":max_price",$max_price);
		$stmt->execute();
		$smart_phones = $stmt->fetchAll(PDO::FETCH_OBJ);
		

		$count_phones = count($smart_phones);

		if($count_phones  > 0){
			
			//Get images for these phones

			for($j=0;$j<$count_phones;$j++){

				$phone_id = $smart_phones[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$smart_phones[$j]->image_name = $image_name;
			}

			echo '{"smartphones":'. json_encode($smart_phones) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null;
		 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}

}










/**
 * Function Name: searchSmartPhonesByOS
 *
 * Returns all phones on search by OS
 *
 * @param (array) (os_id_list) this is an array of the OS IDs
 * @return (object) ($smartphones) an array of the phones that fall in the OS IDs
 */
function searchSmartPhonesByOS() {	

	$request = \Slim\Slim::getInstance()->request();
	$os_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$os_id_list = $os_details['oses'];
	$count = count($os_id_list);	

	try {
		
		$db = getConnection();

		$object_array = array();

		for($i=0;$i<$count;$i++){

			$id = $os_id_list[$i];
			$object_array[] = getSmartPhoneByOS($id,$db);

		}

		$count_phones = count($object_array);

		if($count_phones  > 0){

			$phones_array = call_user_func_array('array_merge', $object_array);			
			
			//Get images for these phones

			$total_phones = count($phones_array);

			for($j=0;$j<$total_phones;$j++){

				$phone_id = $phones_array[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$phones_array[$j]->image_name = $image_name;
			}


			echo '{"smartphones":'. json_encode($phones_array) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		} 

		$db = null;
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}










/**
 * Function Name: searchSmartPhonesByManufacturer
 *
 * Returns all phones on search by manufacturer
 *
 * @param (array) (manufacturer_id_list) this is an array of the manufacturer 
 * @return (object) ($smartphones) an array of the phones that fall in that price range
 */
function searchSmartPhonesByManufacturer() {	

	$request = \Slim\Slim::getInstance()->request();
	$man_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$manufacturer_id_list = $man_details['manufacturers'];
	$count = count($manufacturer_id_list);	

	try {
		
		$db = getConnection();

		$object_array = array();
		
		for($i=0;$i<$count;$i++){

			$id = $manufacturer_id_list[$i];
			$object_array[] = getSmartPhoneByManufacturer($id,$db);

		}

		$count_phones = count($object_array);

		if($count_phones  > 0){

			$phones_array = call_user_func_array('array_merge', $object_array);			
			
			//Get images for these phones

			$total_phones = count($phones_array);

			for($j=0;$j<$total_phones;$j++){

				$phone_id = $phones_array[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$phones_array[$j]->image_name = $image_name;
			}


			echo '{"smartphones":'. json_encode($phones_array) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null;
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}








/**
 * Function Name: searchSmartPhonesByDeviceType
 *
 * Returns all phones on search by device type
 *
 * @param (array) ($device_type_id_list) this is an array of the device type Ids
 * @return (object) ($smartphones) an array of the phones that fall in that price range
 */
function searchSmartPhonesByDeviceType() {	

	$request = \Slim\Slim::getInstance()->request();
	$device_type_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$device_type_id_list = $device_type_details['device_types'];
	$count = count($device_type_id_list);	

	try {
		
		$db = getConnection();

		$object_array = array();
		
		for($i=0;$i<$count;$i++){

			$id = $device_type_id_list[$i];
			$object_array[] = getSmartPhoneByDeviceType($id,$db);

		}

		$count_phones = count($object_array);

		if($count_phones  > 0){

			$phones_array = call_user_func_array('array_merge', $object_array);			
			
			//Get images for these phones

			$total_phones = count($phones_array);

			for($j=0;$j<$total_phones;$j++){

				$phone_id = $phones_array[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$phones_array[$j]->image_name = $image_name;
			}


			echo '{"smartphones":'. json_encode($phones_array) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null;
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}






/**
 * Function Name: searchSmartPhonesByKeyword
 *
 * Returns all phones on search by keyword
 *
 * @param (number) (prices) this is an array of the min and max price range
 * @return (object) ($smartphones) an array of the phones that fall in that price range
 */
function searchSmartPhonesByKeyword() {	

	$request = \Slim\Slim::getInstance()->request();
	$search_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$keyword = $search_details['keyword'];
	
	try {	

		$db = getConnection();	
		
		$smart_phones = getSmartPhoneByKeyword($keyword,$db);	

		$count_phones = count($smart_phones);	

		if($count_phones  > 0){
						
			//Get images for these phones

			for($j=0;$j<$count_phones;$j++){

				$phone_id = $smart_phones[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$smart_phones[$j]->image_name = $image_name;
			}

			echo '{"smartphones":'. json_encode($smart_phones) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null; 
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}





/**
 * Function Name: getSmartPhoneByKeyword
 *
 * Returns all phones on search by keyword
 *
 * @param (number) (prices) this is an array of the min and max price range
 * @return (object) ($smartphones) an array of the phones that fall in that price range
 */
function getSmartPhoneByKeyword($keyword) {	

	$sql_name ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones as main_table INNER JOIN manufacturers as man ON man.man_id = main_table.smart_manufacturers INNER JOIN os_types as ost ON ost.os_id = main_table.smart_os WHERE smart_name REGEXP :keyword ORDER BY sma_id";

	$sql_manufacturer ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones as main_table INNER JOIN manufacturers as man ON man.man_id = main_table.smart_manufacturers INNER JOIN os_types as ost ON ost.os_id = main_table.smart_os WHERE manufacturer_name REGEXP :keyword ORDER BY sma_id";

	$sql_osname ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones as main_table INNER JOIN manufacturers as man ON man.man_id = main_table.smart_manufacturers INNER JOIN os_types as ost ON ost.os_id = main_table.smart_os WHERE os_name REGEXP :keyword ORDER BY sma_id";
	
	$object_array = array();
	$result_array = array();
	$sql_array = array();

	$sql_array[0] = $sql_name;
	$sql_array[1] = $sql_manufacturer;
	$sql_array[2] = $sql_osname;
	
	try {
		$db = getConnection();
		
		$count = count($sql_array);

		for($i=0;$i<$count;$i++){

			$stmt = $db->prepare($sql_array[$i]);  
			$stmt->bindParam(":keyword",$keyword);		
			$stmt->execute();
			$phones = $stmt->fetchAll(PDO::FETCH_OBJ);

			$len = count($phones);

			if($len > 0){
				$object_array[] = $phones;
			}			
		}

		$db = null;		

		$count_phones = count($object_array);

		if($count_phones  > 0){

			//merge all the arrays 
			$phones_array = call_user_func_array('array_merge', $object_array);

			//eliminate duplicates
			$result_array = array_unique($phones_array, SORT_REGULAR);

		}

		return $result_array;

	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}

}




/**
 * Function Name: getSmartPhonesImages
 *
 * Returns all phones on search
 *
 * @param (number) (ids) this is id array of the resource to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function getSmartPhonesImages() {	

	$request = \Slim\Slim::getInstance()->request();
	$phone_details = json_decode($request->getBody(),true);//true makes the result as an array object	
	
	$id_list = $phone_details['ids'];
	$count = count($id_list);	

	try {

		$db = getConnection();

		$object_array = array();

		for($i=0;$i<$count;$i++){

			$id = $id_list[$i];
			$object_array[] = getSmartPhoneImageInner($id,$db);

		}
		
		$db = null;

		echo '{"phoneimages":'. json_encode($object_array) .'}'; 
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}







/**
 * Function Name: deleteSelected
 *
 * Deletes a single patient
 *
 * @param (number) (ids) this is id array of the resource to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteSelected() {	

	$request = \Slim\Slim::getInstance()->request();
	$details = json_decode($request->getBody(),true);//true makes the result as an array object

	if($_SESSION['user_category'] != 2){
			echo '{"error": "Sorry you dont have permission to delete a record!"}';
	}else{
		
		$table = $details['table'];
		$id_list = $details['ids'];		
		
		$count = count($id_list);//the length of array
		
		$total_result = 0;

		try {

			$db = getConnection();

			for($i=0;$i<$count;$i++){

				$id = $id_list[$i];	

				if($table == 'smart_phones'){
					$sql = "DELETE FROM $table WHERE sma_id=:id";
				}else if($table == 'hardwares'){
					$sql = "DELETE FROM $table WHERE had_id=:id";
				}else if($table == 'displays'){
					$sql = "DELETE FROM $table WHERE dis_id=:id";
				}else if($table == 'dimensions'){
					$sql = "DELETE FROM $table WHERE dim_id=:id";
				}else if($table == 'cameras'){
					$sql = "DELETE FROM $table WHERE cam_id=:id";
				}else if($table == 'connectivity'){
					$sql = "DELETE FROM $table WHERE con_id=:id";
				}else if($table == 'technology'){
					$sql = "DELETE FROM $table WHERE tech_id=:id";
				}else if($table == 'smart_phone_accessories'){
					$sql = "DELETE FROM $table WHERE acc_id=:id";
				}	
					
				$stmt = $db->prepare($sql);
				$stmt->bindParam("id", $id);		
				$stmt->execute();
				$rows = $stmt->rowCount();						
				$total_result = $total_result + $rows;
			}

			$db = null;
			
			if($total_result > 0){				
			    echo '{"message":'. json_encode($total_result . ' '. $table .' Details Deleted') .'}';
			}

		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}//
	}
	
}








/**
 * Function Name: deleteSmartPhone
 *
 * Deletes a single smartphone
 *
 * @param (number) (id) this is the id of the smartphone to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteSmartPhone($id) {	
	
	$sql = "DELETE FROM smart_phones WHERE sma_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete a smartphone!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){	
				$num_images_deleted = deleteSmartPhoneImages($id);			
			    echo '{"message":'. json_encode($rows .' Smartphone Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}






/**
 * Function Name: deleteSmartPhoneImages
 *
 * Deletes all images that belong to a certain phone ID
 *
 * @param (number) (id) this is the id of the smartphone 
 * @return (number) (rows) the number of db rows deleted
 */
function deleteSmartPhoneImages($id) {	
	
	$sql = "DELETE FROM smart_phone_images WHERE device_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete a smartphone!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;

			return $rows;
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}




/**
 * Function Name: getSmartPhoneImages
 *
 * Returns all images location for a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smart phone
 * @return (object) ($images) the actual images details
 */
function getSmartPhoneImages($id) {	
	
	$sql ="SELECT img_id, image_name FROM smart_phone_images WHERE device_id = :id ORDER BY img_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$images = $stmt->fetchAll(PDO::FETCH_OBJ);  
		$db = null;
		echo '{"images":'. json_encode($images) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}





/**
 * Function Name: getSmartPhoneImageInner
 *
 * Sub function returns an image location for a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smart phone
 * @return (object) ($image) the actual image details
 */
function getSmartPhoneImageInner($id, $db) {	
	
	$sql ="SELECT image_name FROM smart_phone_images WHERE device_id = :id ORDER BY img_id LIMIT 1";

	try {
		
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$image = $stmt->fetchObject();  
		
		return $image;
	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}




/**
 * Function Name: getAllSmartPhoneImages
 *
 * Returns one image location for each smartphone in the database *
 * 
 * @return (object) ($images) the actual images details
 */
function getAllSmartPhoneImages() {	
	
	$sql ="SELECT * FROM smart_phone_images GROUP BY device_id ORDER BY device_id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);		
		$stmt->execute();
		$images = $stmt->fetchAll(PDO::FETCH_OBJ);  
		$db = null;
		echo '{"images":'. json_encode($images) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}




/**
 * Function Name: getSmartPhoneDetails
 *
 * Returns details of a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smartphone
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneDetails($id) {	
	
	$sql ="SELECT sma_id, smart_type, smart_manufacturers, smart_os, smart_name, smart_memory_size, smart_current_price, smart_previous_price, smart_bonga_points, smart_cpu, smart_card_slot, smart_display_type, smart_display_size, smart_display_protection, smart_network, smart_front_camera, smart_back_camera, smart_connection, smart_sim, smart_badge, manufacturer_name, os_name, device_type_name FROM smart_phones as main_table INNER JOIN device_types as dvt ON dvt.dev_type_id = main_table.smart_type INNER JOIN manufacturers as man ON man.man_id = main_table.smart_manufacturers INNER JOIN os_types as ost ON ost.os_id = main_table.smart_os WHERE main_table.sma_id = :id";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$smart_phone = $stmt->fetchObject();  
		$db = null;
		echo '{"smartphone":'. json_encode($smart_phone) .'}'; 	 
	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}







/**
 * Function Name: getSmartPhone
 *
 * Returns details of a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smartphone
 * @return (object) ($user) the actual user details
 */
function getSmartPhone($id) {
	
 	$sql ="SELECT sma_id, smart_type, smart_manufacturers, smart_os, smart_name, smart_memory_size, smart_current_price, smart_previous_price, smart_bonga_points, smart_cpu, smart_card_slot, smart_display_type, smart_display_size, smart_display_protection, smart_network, smart_front_camera, smart_back_camera, smart_connection, smart_sim, smart_badge FROM smart_phones as p WHERE p.sma_id = :id ";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$smart_phone = $stmt->fetchObject();  
		$db = null;
		echo '{"smartphone":'. json_encode($smart_phone) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}





/**
 * Function Name: getSmartPhoneByDeviceCategory
 *
 * Returns all phones under a certain device type category
 *
 * @param (number) (id) this is the id of the device type
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneByDeviceCategory($id) {	

	$sql ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones WHERE smart_type =:id";

	try {
		
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();		
		$phones_array = $stmt->fetchAll(PDO::FETCH_OBJ);	

		$count_phones = count($phones_array);

		if($count_phones  > 0){				
			
			//Get images for these phones

			$total_phones = count($phones_array);

			for($j=0;$j<$total_phones;$j++){

				$phone_id = $phones_array[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$phones_array[$j]->image_name = $image_name;
			}


			echo '{"smartphones":'. json_encode($phones_array) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null;
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}







/**
 * Function Name: getSmartPhoneInner
 *
 * Sub function returns details of a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smartphone
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneInner($id) {
	

	$sql ="SELECT sma_id, smart_type, smart_manufacturers, smart_os, smart_name, smart_memory_size, smart_current_price, smart_previous_price, smart_bonga_points, smart_cpu, smart_card_slot, smart_display_type, smart_display_size, smart_display_protection, smart_network, smart_front_camera, smart_back_camera, smart_connection, smart_sim, smart_badge, manufacturer_name, os_name, device_type_name FROM smart_phones as main_table INNER JOIN device_types as dvt ON dvt.dev_type_id = main_table.smart_type INNER JOIN manufacturers as man ON man.man_id = main_table.smart_manufacturers INNER JOIN os_types as ost ON ost.os_id = main_table.smart_os WHERE main_table.sma_id = :id";		 

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$smart_phone = $stmt->fetchObject();  
		$db = null;
		return $smart_phone;		 
	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}






/**
 * Function Name: getSmartPhoneByOS
 *
 * Sub function returns details of smartphones under a OS id
 *
 * @param (number) (id) this is the id of the smartphone
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneByOS($id,$db) {	
	
	$sql ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones WHERE smart_os =:id";

	try {
		
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();		
		$smart_phones = $stmt->fetchAll(PDO::FETCH_OBJ);  
		
		return $smart_phones;

	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}






/**
 * Function Name: getSmartPhoneByManufacturer
 *
 * Sub function returns details of smartphones under a manufacturer id
 *
 * @param (number) (id) this is the id of the manufacturer
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneByManufacturer($id,$db) {	
	
	$sql ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones WHERE smart_manufacturers =:id";

	try {
		
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();		
		$smart_phones = $stmt->fetchAll(PDO::FETCH_OBJ);  
		
		return $smart_phones;

	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}





/**
 * Function Name: getSmartPhoneByDeviceType
 *
 * Sub function returns details of smartphones under a device type id
 *
 * @param (number) (id) this is the id of the device type
 * @return (object) ($user) the actual user details
 */
function getSmartPhoneByDeviceType($id,$db) {	
	
	$sql ="SELECT sma_id, smart_name, smart_current_price, smart_bonga_points, smart_previous_price, smart_badge FROM smart_phones WHERE smart_type =:id";

	try {
		
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();		
		$smart_phones = $stmt->fetchAll(PDO::FETCH_OBJ);  
		
		return $smart_phones;

	} catch(PDOException $e) {
		return '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}







/**
 * Function Name: getAllSmartPhones
 *
 * Returns details of a smartphone under a certain id
 *
 * @param (number) (id) this is the id of the smartphone
 * @return (object) ($user) the actual user details
 */
function getAllSmartPhones() {	
	
	$sql ="SELECT sma_id, smart_type, smart_manufacturers, smart_os, smart_name, smart_memory_size, smart_current_price, smart_previous_price, smart_bonga_points, smart_cpu, smart_card_slot, smart_display_type, smart_display_size, smart_display_protection, smart_network, smart_front_camera, smart_back_camera, smart_connection, smart_sim, smart_badge FROM smart_phones ORDER BY smart_type";

	//$object_array = array();

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();		
		$smart_phones = $stmt->fetchAll(PDO::FETCH_OBJ);  
		

		$count_phones = count($smart_phones);

		if($count_phones  > 0){

			//$phones_array = call_user_func_array('array_merge', $smart_phones);			
			
			//Get images for these phones

			$total_phones = count($smart_phones);

			for($j=0;$j<$total_phones;$j++){

				$phone_id = $smart_phones[$j]->sma_id;
				$image_object = getSmartPhoneImageInner($phone_id,$db);
				$image_name = $image_object->image_name;
				$smart_phones[$j]->image_name = $image_name;
			}
		

			echo '{"smartphones":'. json_encode($smart_phones) .'}';

		}else{
			echo '{"smartphones":'. json_encode(false) .'}';
		}

		$db = null;
		
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}








/**
 * Function Name: addSmartPhone
 *
 * Adds a new smartphone 
 *
 * @param (array) ([smartphone details]) the smartphone details
 * @return (number) ($rows) the number of rows added
 *
 */
function addSmartPhone() {
	
	$request = \Slim\Slim::getInstance()->request();
	$smart_phone = json_decode($request->getBody());

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add a new smartphone!"}';
	}else{
			
		$sql = "INSERT INTO smart_phones (smart_type, smart_manufacturers, smart_os, smart_dimensions, smart_hardware, smart_display, smart_cameras, smart_sim, smart_technology, smart_connectivity, smart_category, smart_name, smart_weight, smart_memory_size, smart_battery_capacity, smart_battery_talk_time, smart_current_price, smart_previous_price, smart_badge, smart_description, smart_key_features) VALUES (:smart_type, :smart_manufacturers, :smart_os, :smart_dimensions, :smart_hardware, :smart_display, :smart_cameras, :smart_sim, :smart_technology, :smart_connectivity, :smart_category, :smart_name, :smart_weight, :smart_memory_size, :smart_battery_capacity, :smart_battery_talk_time, :smart_current_price, :smart_previous_price, :smart_badge, :smart_description, :smart_key_features)";
	    
	    $phone_name = cleanField($smart_phone->phone_name);
	    $phone_weight = cleanField($smart_phone->smart_weight);
	    $phone_mem_size = cleanField($smart_phone->smart_memory_size);
	    $phone_batt_cap = cleanField($smart_phone->smart_battery_capacity);
	    $phone_batt_talk_time = cleanField($smart_phone->smart_battery_talk_time);
	    $phone_current_price = cleanField($smart_phone->smart_current_price);
	    $phone_previous_price = cleanField($smart_phone->smart_previous_price);
	    $phone_badge = cleanField($smart_phone->smart_badge);

	    $phone_description = cleanField($smart_phone->smart_description);
	    $phone_features = cleanField($smart_phone->smart_key_features);	    

	    try {
		    
		    
			$flag = 'smartphone';		    
		    if(exists($phone_name, $flag)){
				echo '{"error":'. json_encode('Smartphone Name is not Available, another smartphone has a similar name') .'}';
		    }else{
		    	
		    	$db = getConnection();
				$stmt = $db->prepare($sql);				
				
				$stmt->bindParam("smart_type", $smart_phone->smart_type);
				$stmt->bindParam("smart_manufacturers", $smart_phone->smart_manufacturers);
				$stmt->bindParam("smart_os", $smart_phone->smart_os);
				$stmt->bindParam("smart_dimensions", $smart_phone->smart_dimensions);  
				$stmt->bindParam("smart_hardware", $smart_phone->smart_hardware);
				$stmt->bindParam("smart_display", $smart_phone->smart_display);
				$stmt->bindParam("smart_cameras", $smart_phone->smart_cameras);
				$stmt->bindParam("smart_sim", $smart_phone->smart_sim);
				$stmt->bindParam("smart_technology", $smart_phone->smart_technology);
				$stmt->bindParam("smart_connectivity", $smart_phone->smart_connectivity);
				$stmt->bindParam("smart_category", $smart_phone->smart_category);
				$stmt->bindParam("smart_current_price", $phone_current_price);
				$stmt->bindParam("smart_previous_price", $phone_previous_price);
				$stmt->bindParam("smart_badge", $phone_badge);
				$stmt->bindParam("smart_description", $phone_description);
				$stmt->bindParam("smart_key_features", $phone_features);
				$stmt->bindParam("smart_name", $phone_name);
				$stmt->bindParam("smart_weight", $phone_weight);
				$stmt->bindParam("smart_memory_size", $phone_mem_size);
				$stmt->bindParam("smart_battery_capacity", $phone_batt_cap);
				$stmt->bindParam("smart_battery_talk_time", $phone_batt_talk_time);
				
				$rows = $stmt->execute();
				
				$db = null;
				
				if($rows > 0){
				    echo '{"message":'. json_encode($rows .' New Smartphone Added').'}';			
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: updateSmartPhone
 *
 * Updates a single smartphone 
 *
 * @param (array) ([smartphone details]) the smartphone details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateSmartPhone($id) {

	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$smart_phone = json_decode($body);

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to edit a smartphone!"}';
	}else{			
		
		$sql = "UPDATE smart_phones SET smart_type =:smart_type, smart_manufacturers =:smart_manufacturers, smart_os =:smart_os, smart_dimensions =:smart_dimensions, smart_hardware =:smart_hardware, smart_display =:smart_display, smart_cameras =:smart_cameras, smart_sim =:smart_sim, smart_technology =:smart_technology, smart_connectivity =:smart_connectivity, smart_category =:smart_category, smart_name =:smart_name, smart_weight =:smart_weight, smart_memory_size =:smart_memory_size, smart_battery_capacity =:smart_battery_capacity, smart_battery_talk_time =:smart_battery_talk_time, smart_current_price =:smart_current_price, smart_previous_price =:smart_previous_price, smart_badge =:smart_badge, smart_description =:smart_description, smart_key_features =:smart_key_features WHERE sma_id =:id";

	    $phone_name = cleanField($smart_phone->phone_name);
	    $phone_weight = cleanField($smart_phone->smart_weight);
	    $phone_mem_size = cleanField($smart_phone->smart_memory_size);
	    $phone_batt_cap = cleanField($smart_phone->smart_battery_capacity);
	    $phone_batt_talk_time = cleanField($smart_phone->smart_battery_talk_time);
	    $phone_current_price = cleanField($smart_phone->smart_current_price);
	    $phone_previous_price = cleanField($smart_phone->smart_previous_price);
	    $phone_badge = cleanField($smart_phone->smart_badge);
	    $phone_description = cleanField($smart_phone->smart_description);
	    $phone_features = cleanField($smart_phone->smart_key_features);	

	    try {
		    
		    
			$flag = 'smartphone';		    
		    if(updateExists($phone_name, $id, $flag)){
				echo '{"error":'. json_encode('Smartphone Name is not Available, another smartphone has a similar name') .'}';
		    }else{
		    	
		    	$db = getConnection();
				$stmt = $db->prepare($sql);	

				$stmt->bindParam("id", $id);
				$stmt->bindParam("smart_type", $smart_phone->smart_type);
				$stmt->bindParam("smart_manufacturers", $smart_phone->smart_manufacturers);
				$stmt->bindParam("smart_os", $smart_phone->smart_os);
				$stmt->bindParam("smart_dimensions", $smart_phone->smart_dimensions);  
				$stmt->bindParam("smart_hardware", $smart_phone->smart_hardware);
				$stmt->bindParam("smart_display", $smart_phone->smart_display);
				$stmt->bindParam("smart_cameras", $smart_phone->smart_cameras);
				$stmt->bindParam("smart_sim", $smart_phone->smart_sim);
				$stmt->bindParam("smart_technology", $smart_phone->smart_technology);
				$stmt->bindParam("smart_category", $smart_phone->smart_category);
				$stmt->bindParam("smart_current_price", $phone_current_price);
				$stmt->bindParam("smart_previous_price", $phone_previous_price);
				$stmt->bindParam("smart_badge", $phone_badge);
				$stmt->bindParam("smart_description", $phone_description);
				$stmt->bindParam("smart_key_features", $phone_features);
				$stmt->bindParam("smart_connectivity", $smart_phone->smart_connectivity);
				$stmt->bindParam("smart_name", $phone_name);
				$stmt->bindParam("smart_weight", $phone_weight);
				$stmt->bindParam("smart_memory_size", $phone_mem_size);
				$stmt->bindParam("smart_battery_capacity", $phone_batt_cap);
				$stmt->bindParam("smart_battery_talk_time", $phone_batt_talk_time);
				
				$rows = $stmt->execute();
				
				$db = null;
				
				if($rows > 0){				    
				    echo '{"message":'. json_encode('Smartphone details Updated!') .'}';			
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: addHardware
 *
 * Adds a new hardware 
 *
 * @param (array) ([hardware details]) the hardware details
 * @return (number) ($rows) the number of rows added
 *
 */
function addHardware() {
	
	$request = \Slim\Slim::getInstance()->request();
	$hardware = json_decode($request->getBody());

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add a new hardware!"}';
	}else{
			
		$sql = "INSERT INTO hardwares (system_chip, processor_name, processor_speed, graphics_processor, system_memory, built_in_storage, storage_expansion) VALUES (:system_chip, :processor_name, :processor_speed, :graphics_processor, :system_memory, :built_in_storage, :storage_expansion)";
	    
	    $system_chip = cleanField($hardware->system_chip);
	    $processor_name = cleanField($hardware->processor_name);
	    $processor_speed = cleanField($hardware->processor_speed);
	    $system_memory = cleanField($hardware->system_memory);
	    $built_in_storage = cleanField($hardware->built_in_storage);
	    $storage_expansion = cleanField($hardware->storage_expansion);

	    try { 

	    	$db = getConnection();
			$stmt = $db->prepare($sql);				
			
			$stmt->bindParam("system_chip", $system_chip);
			$stmt->bindParam("processor_name", $processor_name);
			$stmt->bindParam("processor_speed", $processor_speed);
			$stmt->bindParam("graphics_processor", $hardware->graphics_processor);  
			$stmt->bindParam("system_memory", $system_memory);
			$stmt->bindParam("built_in_storage", $built_in_storage);
			$stmt->bindParam("storage_expansion", $storage_expansion);	
			
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){
			    echo '{"message":'. json_encode($rows .' New Hardware details Added').'}';			
			}	    
		   
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: updateHardware
 *
 * Updates a single hardware 
 *
 * @param (array) ([hardware details]) the hardware details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateHardware($id) {

	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$hardware = json_decode($body);

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to edit a hardware!"}';
	}else{			
		
		$sql = "UPDATE hardwares SET system_chip =:system_chip, processor_name =:processor_name, processor_speed =:processor_speed, graphics_processor =:graphics_processor, system_memory =:system_memory, built_in_storage =:built_in_storage, storage_expansion =:storage_expansion WHERE had_id =:id";

	    $system_chip = cleanField($hardware->system_chip);
	    $processor_name = cleanField($hardware->processor_name);
	    $processor_speed = cleanField($hardware->processor_speed);
	    $system_memory = cleanField($hardware->system_memory);
	    $built_in_storage = cleanField($hardware->built_in_storage);
	    $storage_expansion = cleanField($hardware->storage_expansion);

	    try {
		    
		    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);	

			$stmt->bindParam("id", $id);
			$stmt->bindParam("system_chip", $system_chip);
			$stmt->bindParam("processor_name", $processor_name);
			$stmt->bindParam("processor_speed", $processor_speed);
			$stmt->bindParam("graphics_processor", $hardware->graphics_processor);  
			$stmt->bindParam("system_memory", $system_memory);
			$stmt->bindParam("built_in_storage", $built_in_storage);
			$stmt->bindParam("storage_expansion", $storage_expansion);			
			
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){				    
			    echo '{"message":'. json_encode('Hardware details Updated!') .'}';			
			}	    
		    
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}







/**
 * Function Name: deleteHardware
 *
 * Deletes a single hardware
 *
 * @param (number) (id) this is the id of the hardware to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteHardware($id) {	
	
	$sql = "DELETE FROM hardwares WHERE had_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete a hardware!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){				
			    echo '{"message":'. json_encode($rows .' Hardware Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}









/**
 * Function Name: addTechnology
 *
 * Adds a new technology 
 *
 * @param (array) ([technology details]) the technology details
 * @return (number) ($rows) the number of rows added
 *
 */
function addTechnology() {
	
	$request = \Slim\Slim::getInstance()->request();
	$technology = json_decode($request->getBody());

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add technology details!"}';
	}else{
			
		$sql = "INSERT INTO technology (technology_gsm, technology_utms, technology_data, technology_positioning, technology_navigation) VALUES (:technology_gsm, :technology_utms, :technology_data, :technology_positioning, :technology_navigation)";
	    
	    $technology_gsm = cleanField($technology->technology_gsm);
	    $technology_utms = cleanField($technology->technology_utms);
	    $technology_data = cleanField($technology->technology_data);
	    $technology_positioning = cleanField($technology->technology_positioning);
	    $technology_navigation = cleanField($technology->technology_navigation);
	    
	    try { 

	    	$db = getConnection();
			$stmt = $db->prepare($sql);				
			
			$stmt->bindParam("technology_gsm", $technology_gsm);
			$stmt->bindParam("technology_utms", $technology_utms);
			$stmt->bindParam("technology_data", $technology_data);
			$stmt->bindParam("technology_positioning", $technology_positioning);  
			$stmt->bindParam("technology_navigation", $technology_navigation);
					
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){
			    echo '{"message":'. json_encode($rows .' New Technology details Added').'}';			
			}	    
		   
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: updateTechnology
 *
 * Updates a single technology 
 *
 * @param (array) ([technology details]) the technology details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateTechnology($id) {

	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$technology = json_decode($body);

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to edit technology details!"}';
	}else{			
		
		$sql = "UPDATE technology SET technology_gsm =:technology_gsm, technology_utms =:technology_utms, technology_data =:technology_data, technology_positioning =:technology_positioning, technology_navigation =:technology_navigation WHERE tech_id =:id";

	    $technology_gsm = cleanField($technology->technology_gsm);
	    $technology_utms = cleanField($technology->technology_utms);
	    $technology_data = cleanField($technology->technology_data);
	    $technology_positioning = cleanField($technology->technology_positioning);
	    $technology_navigation = cleanField($technology->technology_navigation);

	    try {
		    
		    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);	

			$stmt->bindParam("id", $id);
			$stmt->bindParam("technology_gsm", $technology_gsm);
			$stmt->bindParam("technology_utms", $technology_utms);
			$stmt->bindParam("technology_data", $technology_data);
			$stmt->bindParam("technology_positioning", $technology_positioning);  
			$stmt->bindParam("technology_navigation", $technology_navigation);	
			
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){				    
			    echo '{"message":'. json_encode('Technology details Updated!') .'}';			
			}	    
		    
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}







/**
 * Function Name: deleteTechnology
 *
 * Deletes a single technology
 *
 * @param (number) (id) this is the id of the technology to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteTechnology($id) {	
	
	$sql = "DELETE FROM technology WHERE tech_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete technology details!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){				
			    echo '{"message":'. json_encode($rows .' Technology Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}









/**
 * Function Name: addConnectivity
 *
 * Adds a new connectivity 
 *
 * @param (array) ([connectivity details]) the connectivity details
 * @return (number) ($rows) the number of rows added
 *
 */
function addConnectivity() {
	
	$request = \Slim\Slim::getInstance()->request();
	$connectivity = json_decode($request->getBody());

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add connectivity details!"}';
	}else{
			
		$sql = "INSERT INTO connectivity (connectivity_bluetooth, connectivity_wifi, connectivity_usb, connectivity_other) VALUES (:connectivity_bluetooth, :connectivity_wifi, :connectivity_usb, :connectivity_other)";
	    
	    $connectivity_bluetooth = cleanField($connectivity->connectivity_bluetooth);
	    $connectivity_wifi = cleanField($connectivity->connectivity_wifi);
	    $connectivity_usb = cleanField($connectivity->connectivity_usb);
	    $connectivity_other = cleanField($connectivity->connectivity_other);
	    
	    try { 

	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("connectivity_bluetooth", $connectivity_bluetooth);
			$stmt->bindParam("connectivity_wifi", $connectivity_wifi);
			$stmt->bindParam("connectivity_usb", $connectivity_usb);
			$stmt->bindParam("connectivity_other", $connectivity_other);  
					
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){
			    echo '{"message":'. json_encode($rows .' New Connectivity details Added').'}';			
			}	    
		   
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}








/**
 * Function Name: updateConnectivity
 *
 * Updates a single connectivity 
 *
 * @param (array) ([connectivity details]) the connectivity details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateConnectivity($id) {

	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$connectivity = json_decode($body);

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to edit connectivity details!"}';
	}else{			
		
		$sql = "UPDATE connectivity SET connectivity_bluetooth =:connectivity_bluetooth, connectivity_wifi =:connectivity_wifi, connectivity_usb =:connectivity_usb, connectivity_other =:connectivity_other WHERE con_id =:id";

	    $connectivity_bluetooth = cleanField($connectivity->connectivity_bluetooth);
	    $connectivity_wifi = cleanField($connectivity->connectivity_wifi);
	    $connectivity_usb = cleanField($connectivity->connectivity_usb);
	    $connectivity_other = cleanField($connectivity->connectivity_other);
	    
	    try {
		    
		    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);	

			$stmt->bindParam("id", $id);
			$stmt->bindParam("connectivity_bluetooth", $connectivity_bluetooth);
			$stmt->bindParam("connectivity_wifi", $connectivity_wifi);
			$stmt->bindParam("connectivity_usb", $connectivity_usb);
			$stmt->bindParam("connectivity_other", $connectivity_other);  
			
			
			$rows = $stmt->execute();
			
			$db = null;
			
			if($rows > 0){				    
			    echo '{"message":'. json_encode('Connectivity details Updated!') .'}';			
			}	    
		    
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}







/**
 * Function Name: deleteConnectivity
 *
 * Deletes a single connectivity
 *
 * @param (number) (id) this is the id of the connectivity to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteConnectivity($id) {	
	
	$sql = "DELETE FROM connectivity WHERE con_id =:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete connectivity details!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){				
			    echo '{"message":'. json_encode($rows .' Connectivity Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}









/**
 * Function Name: addCamera
 *
 * Adds a new camera 
 *
 * @param (array) ([camera details]) the camera details
 * @return (number) ($rows) the number of rows added
 *
 */
function addCamera() {
    
    $request = \Slim\Slim::getInstance()->request();
    $camera = json_decode($request->getBody());

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to add camera details!"}';
    }else{
            
        $sql = "INSERT INTO cameras (camera_back_megapixels, front_camera) VALUES (:camera_back_megapixels, :front_camera)";
        
        $camera_back_megapixels = cleanField($camera->camera_back_megapixels);        
        
        try { 

            $db = getConnection();
            $stmt = $db->prepare($sql);
            
            $stmt->bindParam("camera_back_megapixels", $camera_back_megapixels);
            $stmt->bindParam("front_camera", $camera->front_camera);             
                    
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){
                echo '{"message":'. json_encode($rows .' New Camera details Added').'}';          
            }       
           
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}








/**
 * Function Name: updateCamera
 *
 * Updates a single camera 
 *
 * @param (array) ([camera details]) the camera details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateCamera($id) {

    $request = \Slim\Slim::getInstance()->request();
    $body = $request->getBody();
    $camera = json_decode($body);

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to edit camera details!"}';
    }else{          
        
        $sql = "UPDATE cameras SET camera_back_megapixels =:camera_back_megapixels, front_camera =:front_camera WHERE cam_id =:id";

        $camera_back_megapixels = cleanField($camera->camera_back_megapixels);
        
        try {
            
                
            $db = getConnection();
            $stmt = $db->prepare($sql); 

            $stmt->bindParam("id", $id);
            $stmt->bindParam("camera_back_megapixels", $camera_back_megapixels);
            $stmt->bindParam("front_camera", $camera->front_camera);           
            
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){                  
                echo '{"message":'. json_encode('Camera details Updated!') .'}';          
            }       
            
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}







/**
 * Function Name: deleteCamera
 *
 * Deletes a single camera
 *
 * @param (number) (id) this is the id of the camera to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteCamera($id) {  
    
    $sql = "DELETE FROM cameras WHERE cam_id =:id";

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to delete camera details!"}';
    }else{
        try {
            $db = getConnection();
            $stmt = $db->prepare($sql);
            $stmt->bindParam("id", $id);        
            $stmt->execute();
            $rows = $stmt->rowCount();  
            $db = null;
            
            if($rows > 0){              
                echo '{"message":'. json_encode($rows .' Camera Deleted') .'}';
            }
            
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}';
        }
    }
    
}











/**
 * Function Name: addDimension
 *
 * Adds a new dimension 
 *
 * @param (array) ([dimension details]) the dimension details
 * @return (number) ($rows) the number of rows added
 *
 */
function addDimension() {
    
    $request = \Slim\Slim::getInstance()->request();
    $dimension = json_decode($request->getBody());

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to add dimension details!"}';
    }else{
            
        $sql = "INSERT INTO dimensions (dimension_width, dimension_height, dimension_depth, dimension_type) VALUES (:dimension_width, :dimension_height, :dimension_depth, :dimension_type)";
        
        $dimension_width = cleanField($dimension->dimension_width);
        $dimension_height = cleanField($dimension->dimension_height);
        $dimension_depth = cleanField($dimension->dimension_depth);
        
        try { 

            $db = getConnection();
            $stmt = $db->prepare($sql);
            
            $stmt->bindParam("dimension_width", $dimension_width);
            $stmt->bindParam("dimension_height", $dimension_height);
            $stmt->bindParam("dimension_depth", $dimension_depth);
            $stmt->bindParam("dimension_type", $dimension->dimension_type);             
                    
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){
                echo '{"message":'. json_encode($rows .' New Dimension details Added').'}';          
            }       
           
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}








/**
 * Function Name: updateDimension
 *
 * Updates a single dimension 
 *
 * @param (array) ([dimension details]) the dimension details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateDimension($id) {

    $request = \Slim\Slim::getInstance()->request();
    $body = $request->getBody();
    $dimension = json_decode($body);

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to edit dimension details!"}';
    }else{          
        
        $sql = "UPDATE dimensions SET dimension_width =:dimension_width, dimension_height =:dimension_height, dimension_depth =:dimension_depth, dimension_type =:dimension_type WHERE dim_id =:id";

        $dimension_width = cleanField($dimension->dimension_width);
        $dimension_height = cleanField($dimension->dimension_height);
        $dimension_depth = cleanField($dimension->dimension_depth);
        
        try {            
                
            $db = getConnection();
            $stmt = $db->prepare($sql); 

            $stmt->bindParam("id", $id);
            $stmt->bindParam("dimension_width", $dimension_width);
            $stmt->bindParam("dimension_height", $dimension_height);
            $stmt->bindParam("dimension_depth", $dimension_depth);
            $stmt->bindParam("dimension_type", $dimension->dimension_type);           
            
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){                  
                echo '{"message":'. json_encode('Dimension details Updated!') .'}';          
            }       
            
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}







/**
 * Function Name: deleteDimension
 *
 * Deletes a single dimension
 *
 * @param (number) (id) this is the id of the dimension to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteDimension($id) {  
    
    $sql = "DELETE FROM dimensions WHERE dim_id =:id";

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to delete dimension details!"}';
    }else{
        try {
            $db = getConnection();
            $stmt = $db->prepare($sql);
            $stmt->bindParam("id", $id);        
            $stmt->execute();
            $rows = $stmt->rowCount();  
            $db = null;
            
            if($rows > 0){              
                echo '{"message":'. json_encode($rows .' Dimension Details Deleted') .'}';
            }
            
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}';
        }
    }
    
}










/**
 * Function Name: addDisplay
 *
 * Adds a new display 
 *
 * @param (array) ([display details]) the display details
 * @return (number) ($rows) the number of rows added
 *
 */
function addDisplay() {
    
    $request = \Slim\Slim::getInstance()->request();
    $display = json_decode($request->getBody());

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to add display details!"}';
    }else{
            
        $sql = "INSERT INTO displays (display_size, display_resolution, display_technology) VALUES (:display_size, :display_resolution, :display_technology)";
        
        $display_size = cleanField($display->display_size);
        $display_resolution = cleanField($display->display_resolution);
        $display_technology = cleanField($display->display_technology);
        
        try { 

            $db = getConnection();
            $stmt = $db->prepare($sql);
            
            $stmt->bindParam("display_size", $display_size);            
            $stmt->bindParam("display_resolution", $display_resolution);
            $stmt->bindParam("display_technology", $display_technology);            
                    
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){
                echo '{"message":'. json_encode($rows .' New Display details Added').'}';          
            }       
           
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}








/**
 * Function Name: updateDisplay
 *
 * Updates a single display 
 *
 * @param (array) ([display details]) the display details
 * @return (number) ($rows) the number of rows added
 *
 */
function updateDisplay($id) {

    $request = \Slim\Slim::getInstance()->request();
    $body = $request->getBody();
    $display = json_decode($body);

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to edit display details!"}';
    }else{          
        
        $sql = "UPDATE displays SET display_size =:display_size, display_resolution =:display_resolution, display_technology =:display_technology WHERE dis_id =:id";

        $display_size = cleanField($display->display_size);
        $display_resolution = cleanField($display->display_resolution);
        $display_technology = cleanField($display->display_technology);
        
        try {            
                
            $db = getConnection();
            $stmt = $db->prepare($sql); 

            $stmt->bindParam("id", $id);
            $stmt->bindParam("display_size", $display_size);            
            $stmt->bindParam("display_resolution", $display_resolution);
            $stmt->bindParam("display_technology", $display_technology);           
            
            $rows = $stmt->execute();
            
            $db = null;
            
            if($rows > 0){                  
                echo '{"message":'. json_encode('Display details Updated!') .'}';          
            }       
            
        
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}'; 
        }

    }      
    
}







/**
 * Function Name: deleteDisplay
 *
 * Deletes a single display
 *
 * @param (number) (id) this is the id of the display to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteDisplay($id) {  
    
    $sql = "DELETE FROM displays WHERE dis_id =:id";

    if($_SESSION['user_category'] != 2){
        echo '{"error": "Sorry you dont have permission to delete display details!"}';
    }else{
        try {
            $db = getConnection();
            $stmt = $db->prepare($sql);
            $stmt->bindParam("id", $id);        
            $stmt->execute();
            $rows = $stmt->rowCount();  
            $db = null;
            
            if($rows > 0){              
                echo '{"message":'. json_encode($rows .' Display Details Deleted') .'}';
            }
            
        } catch(PDOException $e) {      
            echo '{"error":'. json_encode($e->getMessage()) .'}';
        }
    }
    
}









/**
 * Function Name: exists
 *
 * checks whether a certain record exists in the db
 *
 * @param (text) ($param) the text to be checked whether it exists
 * @param (boolean) ($flag) the flag to differentiate the checks
 * @return (boolean) ($exist) either true or false if it exists
 *
 */
function exists($param, $flag){
    
	$exist = false;
	$db = getDBConnection();
	
	if($flag == 'user'){
	
		$query = "SELECT * FROM app_users WHERE username = '$param'";
		$result = mysqli_query($db,$query);
		 
		while($row = mysqli_fetch_assoc($result)){
			$exist = true;
			break;//get out of the while loop
		}
		
	}else if($flag == 'smartphone'){

		$query = "SELECT * FROM smart_phones WHERE smart_name = '$param'";
		$result = mysqli_query($db,$query);
		 
		while($row = mysqli_fetch_assoc($result)){
			$exist = true;
			break;//get out of the while loop
		}

	}else if($flag == 'accessory'){

		$query = "SELECT * FROM smart_phone_accessories WHERE accessory_name = '$param'";
		$result = mysqli_query($db,$query);
		 
		while($row = mysqli_fetch_assoc($result)){
			$exist = true;
			break;//get out of the while loop
		}

	}else if($flag == 'open_user'){
	
		$dbconn = getDBConnectionTwo();
		$query = "SELECT id, phone_number FROM open_users WHERE phone_number = '$param'";
		
		$result = mysqli_query($dbconn,$query);
					 
		while($row = mysqli_fetch_assoc($result)){
			$exist = true;
			break; //get out of the while loop
		}

		mysqli_close($dbconn);
		
	}
    
    mysqli_close($db);    
    return $exist;
    
}





/**
 * Function Name: updateExists
 *
 * checks whether a certain record exists in the db before an update
 *
 * @param (text) ($param) the text to be checked whether it exists
 * @param (boolean) ($flag) the flag to differentiate the checks
 * @return (boolean) ($exist) either true or false if it exists
 *
 */
function updateExists($param,$id, $flag){

	$exist = false;
	$db = getDBConnection();

	if($flag == 'user'){

		$query = "SELECT * FROM app_users WHERE username = '$param'";
		$result = mysqli_query($db,$query);
			
		while($row = mysqli_fetch_assoc($result)){
			if($row['usid'] != $id){
					$exist = true;
					break;//get out of the while loop
				}
		}

	}else if($flag == 'smartphone'){

		$query = "SELECT * FROM smart_phones WHERE smart_name = '$param'";
		$result = mysqli_query($db,$query);
			
		while($row = mysqli_fetch_assoc($result)){
			if($row['sma_id'] != $id){
					$exist = true;
					break;//get out of the while loop
				}
		}

	}else if($flag == 'accessory'){

		$query = "SELECT * FROM smart_phone_accessories WHERE accessory_name = '$param'";
		$result = mysqli_query($db,$query);
			
		while($row = mysqli_fetch_assoc($result)){
			if($row['acc_id'] != $id){
					$exist = true;
					break;//get out of the while loop
				}
		}

	}

	mysqli_close($db);

	return $exist;

}






/**
 * Function Name: passwordOk
 *
 * verifys if password entered is the one in the db before updating it
 *
 * @param (number) ($id) the id of the admin user
 * @param (text) ($password) this is the current password in text format
 * @return (boolean) ($ok) either true or false if its OK
 *
 */
function passwordOk($id, $password){
	
	$flag = false;
	$ok = false;
	$hasher = new PasswordHash(8, FALSE);
	$hash = getHash($id, $flag);
	
	if($hash != "EMPTY"){
		$check = $hasher->CheckPassword($password, $hash);
		if($check){	
			$ok = true;	
		}		 
	}	

	return $ok;

}




/**
 * Function Name: updatePassword
 *
 * Updates the admin password
 *
 * @param (number) (id) this is the id of the admin user to be updated
 * @return (number) (rows) the number of db rows updated
 */
function updatePassword($id) {	
	
	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$pwset = json_decode($body);

	$sql = "UPDATE app_users SET password =:password WHERE usid =:id";

	try {		
		
		if(passwordOk($id, $pwset->current_password)){
			
			$db = getConnection();
			$stmt = $db->prepare($sql);		

			$hasher = new PasswordHash(8, FALSE);				
			$password = $hasher->HashPassword($pwset->new_password);
			
			$stmt->bindParam("id", $id);
			$stmt->bindParam("password", $password);
			$rows = $stmt->execute();
			$db = null;
			if($rows > 0){
				echo '{"message":'. json_encode('User Password Updated!<br/>Logout and login again to see changes') .'}';
			}
			
		}else{		
			echo '{"error":'. json_encode('Incorrect Current Passowrd') .'}';		
		}

	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}
}






/**
 * Function Name: deleteUser
 *
 * Deletes a single user
 *
 * @param (number) (id) this is the id of the recommendation to be deleted
 * @return (number) (rows) the number of db rows deleted
 */
function deleteUser($id) {	
	
	$sql = "DELETE FROM app_users WHERE usid=:id";

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to delete a user!"}';
	}else{
		try {
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);		
			$stmt->execute();
			$rows = $stmt->rowCount();	
			$db = null;
			
			if($rows > 0){				
			    echo '{"message":'. json_encode($rows .' User Deleted') .'}';
			}
			
		} catch(PDOException $e) {		
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}
	}
	
}







/**
 * Function Name: addUser
 *
 * Adds a new admin user 
 *
 * @param (array) ([user details]) the admin user details
 * @return (number) ($rows) the number of rows added
 *
 */
function addUser() {
	
	$request = \Slim\Slim::getInstance()->request();
	$user = json_decode($request->getBody());

	if($_SESSION['user_category'] != 2){
		echo '{"error": "Sorry you dont have permission to add a new user!"}';
	}else{
			
		$sql = "INSERT INTO app_users (firstname, lastname, username, password, email, ucd) VALUES (:firstname, :lastname, :username, :password, :email, :category)";
	    
	    try {
		    
		    
			$flag = 'user';		    
		    if(exists($user->username, $flag)){
				echo '{"error":'. json_encode('Username NOT Available') .'}';
		    }else{
		    	
		    	$db = getConnection();
				$stmt = $db->prepare($sql);				
				$hasher = new PasswordHash(8, FALSE);				
				$password = $hasher->HashPassword($user->password);
				$stmt->bindParam("firstname", $user->first_name);
				$stmt->bindParam("lastname", $user->last_name);
				$stmt->bindParam("username", $user->username);
				$stmt->bindParam("password", $password);  
				$stmt->bindParam("email", $user->email_address);
				$stmt->bindParam("category", $user->category);
				
				$rows = $stmt->execute();
				
				$db = null;
				if($rows > 0){
				    echo '{"message":'. json_encode($rows .' New Application User Added').'}';			
				}
	    
		    }
	    
	    } catch(PDOException $e) {		
		    echo '{"error":'. json_encode($e->getMessage()) .'}'; 
	    }

	}	   
	
}





/**
 * Function Name: updateUser
 *
 * Updates a single admin user
 *
 * @param (number) (id) this is the id of the admin user to be updated
 * @return (number) (rows) the number of db rows updated
 */
function updateUser($id) {	
	
	$request = \Slim\Slim::getInstance()->request();
	$body = $request->getBody();
	$user = json_decode($body);
	
	$sql = "UPDATE app_users SET username =:username, firstname =:firstname, lastname =:lastname, email =:email WHERE usid =:id";
	
	
	$flag = 'user';
	if(updateExists($user->username, $id, $flag)){
		echo '{"error":'. json_encode('Username NOT Available') .'}';
	}else{
		
		try {
			
			$db = getConnection();
			$stmt = $db->prepare($sql);
			$stmt->bindParam("id", $id);
			$stmt->bindParam("username", $user->username);
			$stmt->bindParam("firstname", $user->first_name);
			$stmt->bindParam("lastname", $user->last_name);
			$stmt->bindParam("email", $user->email_address);
			$rows = $stmt->execute();
			$db = null;
			if($rows > 0){
				echo '{"message":'. json_encode('User Details Updated!<br/>Logout and login again to see changes') .'}';
			}
		
		} catch(PDOException $e) {
			echo '{"error":'. json_encode($e->getMessage()) .'}';
		}		
	}
	
}






/**
 * Function Name: getUsers
 *
 * Retrieves all application users details 
 * 
 * @return (object) ($users) the actual users details
 */
function getUsers() {

	$sql ="SELECT u.usid, u.username, u.firstname, u.lastname, u.email, c.category_name FROM app_users as u INNER JOIN user_categories as c ON c.ucid = u.ucd ORDER BY u.usid";
		
		
	try {
		$db = getConnection();
		$stmt = $db->query($sql);		  
		$users = $stmt->fetchAll(PDO::FETCH_OBJ); 
		$db = null;
		echo '{"users":'. json_encode($users) .'}'; 
		
	} catch(PDOException $e) {		
		echo '{"error":'. json_encode($e->getMessage()) .'}';
	}

}





/**
 * Function Name: getUser
 *
 * Returns details of a users under a certain id
 *
 * @param (number) (id) this is the id of the user
 * @return (object) ($user) the actual user details
 */
function getUser($id) {	
	
	$sql ="SELECT u.usid, u.username, u.firstname, u.lastname, u.email, c.category_name FROM app_users as u INNER JOIN user_categories as c ON c.ucid = u.ucd WHERE u.usid = :id ";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);  
		$stmt->bindParam(":id",$id);
		$stmt->execute();
		$user = $stmt->fetchObject();  
		$db = null;
		echo '{"user":'. json_encode($user) .'}'; 
	} catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
		 
	}
}






/**
 * Function Name: userLogin
 *
 * used to login to the app so it verifys if the entered details are correct
 *
 * @param (text) ($username) the username of the admin user
 * @param (text) ($password) the password of the admin user
 * @return (json) (json_encode($user)) the user details 
 *
 */
function userLogin() {
    
    $request = \Slim\Slim::getInstance()->request();
    $user = json_decode($request->getBody());
    $username = $user->username;
    $password = $user->password;
    
    try {	    
	    
	    $q = findUser($username, $password);
	    
	    if($q == "BAD-USERNAME"){
		
			echo '{"error":'. json_encode('Invalid Username') .'}';			
		
	    }else if($q == "BAD-PASS"){ 
		
			echo '{"error":'. json_encode('Invalid Password') .'}';
		
	    }else{
			
	    	$db = getConnection();
	    	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$stmt = $db->prepare($q);  
			$stmt->bindParam("username", $username);
			$stmt->bindParam("password", $password);
			$stmt->execute();
			$user = $stmt->fetchObject();  
			$db = null;			

			$_SESSION = array();
			$_SESSION['id'] = $user->usid;
			$_SESSION['firstname'] = $user->firstname;
			$_SESSION['lastname'] = $user->lastname;
			$_SESSION['username'] = $user->username;
			$_SESSION['email'] = $user->email;
			$_SESSION['user_category'] = $user->user_category;
			$_SESSION['category_name'] = $user->category_name;
			
			echo '{"user":'. json_encode('OK') .'}';
		
	    }
	    
		    
    } catch(PDOException $e) {
		echo '{"error":'. json_encode($e->getMessage()) .'}';
    }
	
}








/**
 * Function Name: findUser
 *
 * a sub-function of userLogin() provides the sql for a user to login
 *
 * @param (text) ($username) the username of the admin user
 * @param (text) ($password) the password of the admin user
 * @return (sql) ($sql) the sql text that queries the db for the admin user to login
 *
 */
function findUser($username, $password){

	$sql = '';

	if(($username != "") && ($password != "")){

		$flag = true;
		$hasher = new PasswordHash(8, FALSE);
		$hash = getHash($username, $flag);

		
		
		if($hash != "EMPTY"){
				
			$check = $hasher->CheckPassword($password, $hash);
			
			if($check){
					
				$sql = "SELECT u.usid,firstname,lastname,username,email,ucd as user_category,category_name FROM app_users as u INNER JOIN user_categories as c ON c.ucid = u.ucd WHERE username = '$username' AND password = '$hash'";
				
				
			}else{
					
				$sql = "BAD-PASS";					
			}

		}else{

			$sql = "BAD-USERNAME";

		}

	}else{

		throw new Exception("Unsupported property!");

	}

	return $sql;
	
}





/**
 * Function Name: getHash
 *
 * a sub-function of userLogin() provides hash password of an admin user
 *
 * @param (text) ($param) the text to be queried in the db
 * @param (boolean) ($flag) the flag that differentiate what to query for
 * @return (hash) ($hash) the password in hash format
 *
 */
function getHash($param, $flag){
    
	$hash = "EMPTY";
	$db = getDBConnection();
	
    if($flag){
    	
    	$query = "SELECT * FROM app_users WHERE username = '$param'";    	
    	$result = mysqli_query($db,$query);
    	
    	while($row = mysqli_fetch_assoc($result)){
    		$hash = $row['password'];
    		break;//get out of the while loop
    	}    	
    	
    }else{
    	
    	$query = "SELECT * FROM app_users WHERE usid = '$param'";    	
    	$result = mysqli_query($db,$query);
    	
    	
    	while($row = mysqli_fetch_assoc($result)){
    		$hash = $row['password'];
    		break;//get out of the while loop
    	}    	
    	
    }
    
    mysqli_close($db); 
    
    return $hash;
    
}




function cleanField($text){	
	
	$name = preg_replace("/[^a-zA-Z0-9\/_|+,<>() .-]/", '', $text);//remove special characters
	$name = preg_replace("/[\"\']/", '', $name); //remove any quotes

	return $name;
}

function cleanURLField($text){
	
	if(!filter_var($text, FILTER_VALIDATE_URL)){
		$text = 'url not valid';
	}

	return $text;
}





?>
