/**
Custom module for you to write your own javascript functions
**/
var allSmartPhones;
var maxCompare = 3;
var compare = [];
var idleTime = 0;

var Custom = (function () {
  // private functions & variables

  // public functions
  return {
    //main function
    init: function () {
      var host = this.appSettings().api_url;
      this.checkOnline();
    },

    appSettings: function () {
      //var host = "http://41.57.109.170/demos/";
      // var host = "http://localhost/safaricomshop/";
      var host = "https://izoneafrica.net/safshop/";

      var settings = {
        api_url: host + "api/index.php/phonefinder",
        sms_url: host + "api/index.php/openday",
        images_url: host + "uploads/files/",
        accessory_image_url: host + "accessories/files/",
        web_host: host
      };

      return settings;
    },

    //Pull together details of a single smartphone
    showPhone: function (id) {
      var that = this;

      const currentPhone = allSmartPhones.filter((phone) => {
        return phone.sma_id == id;
      });

      if (currentPhone.length > 0) {
        that.showSmartPhone(currentPhone);
      }
    },

    //Returns all specs for all smartphones
    getSmartPhones: function () {
      var host = this.appSettings().api_url;

      var defer = $.Deferred();

      var res = $.ajax({
        type: "GET",
        contentType: "application/json; charset=utf-8",
        url: host + "/smart_phone/",
        dataType: "json",
        cache: false,
        success: function (data, textStatus, jqXHR) {
          var success = data.smartphones;
          var error = data.error;

          if (error !== undefined) {
            console.log(error);
          } else if (success !== undefined) {
            response = success;
            defer.resolve(response);
          } else {
            console.log("Please try again");
          }
        },
        error: function (jqXHR, textStatus, errorThrown) {
          console.log(jqXHR.responseText);
        },
      }); //end of ajax

      return defer.promise();
    },

    //Display all smartphone details on page
    showSmartPhones: function () {
      var that = this;

      var phones = that.getSmartPhones().then(function (phones_respon) {
        var len = phones_respon.length;

        if (len > 0 && len != undefined) {
          allSmartPhones = phones_respon;
          that.showSmartPhonesList(phones_respon);
        }
      });

      return false;
    },

    //Displays all the phones from DB on page
    showSmartPhonesList: function (phones_respon) {
      var len = phones_respon.length;
      var images_location = this.appSettings().images_url;
      var list_item = "";

      var container = $("#phones-container");

      container.empty();
        
      if (len > 0) {

        for (var i = 0; i < len; i++) {
          var item = `
            <div class="col-4 col-sm-4 col-lg-3 item">
              <div class="row">
                <div class="col-md-12">
                  <div class="row">
                    <div class="col-sm-6 pr-0">
                      <img
                        src="assets/img/files/${phones_respon[i].image_name}"
                        alt="${phones_respon[i].smart_name}"
                        class="img-fluid animate__animated animate__bounce animate__infinite animate__slower"
                      />
                    </div>
                    <div class="col-sm-6 pl-0 pt-5">
                      <button class="btn btn-secondary btn-block mb-2" onClick="Custom.addToCompare(${phones_respon[i].sma_id})">
                        Compare <i class="fas fa-copy"></i>
                      </button>
                      <div class="item-description p-2">
                        <h6>${phones_respon[i].smart_name}</h6>
                        <h6>${phones_respon[i].smart_current_price}</h6>
                      </div>
                    </div>
                  </div>
                </div>
                <div class="col-md-12 mt-3 pl-0">
                  <button class="btn btn-secondary btn-block" onClick="Custom.showPhone(${phones_respon[i].sma_id})">View Details</button>
                </div>
              </div>
            </div>
          `;
          container.append(item);
        }
      } else{
          container.html(`<div class="alert alert-warning col-md-12"><p>There are no devices to display</p></div>`);
      }
    },

    //Display a particular smartphone details on page
    showSmartPhone: function (phone_respon_obj) {
      const phone_respon = phone_respon_obj[0];

      $("#modal-phone-name").html(phone_respon.smart_name);

      $("#modal-phone-image").attr(
        "src",
        "assets/img/files/" + phone_respon.image_name
      );

      //System
      $("#modal-phone-os").html(phone_respon.os_name);
      $("#modal-phone-cpu").html(phone_respon.smart_cpu);

      //Display
      $("#modal-phone-type").html(phone_respon.smart_display_type);
      $("#modal-phone-size").html(phone_respon.smart_display_size);
      $("#modal-phone-battery").html(phone_respon.smart_display_protection);

      //Memory
      $("#modal-phone-cpu").html(phone_respon.smart_memory_size);
      $("#modal-phone-card-slot").html(phone_respon.smart_card_slot);

      //Camera
      $("#modal-phone-front").html(phone_respon.smart_front_camera);
      $("#modal-phone-back").html(phone_respon.smart_back_camera);

      $("#modal-phone-network").html(phone_respon.smart_network);
      $("#modal-phone-connection").html(phone_respon.smart_connection);
      $("#modal-phone-sim").html(phone_respon.smart_sim);

      $("#single-phone-modal").modal();

      return false;
    },

    filterManufacturer: function (id) {
      var that = this;

      const filteredPhones = allSmartPhones.filter((phone) => {
        return phone.smart_manufacturers == id;
      });

      that.showSmartPhonesList(filteredPhones);
    },

    filterOs: function (id) {
      var that = this;

      const filteredPhones = allSmartPhones.filter((phone) => {
        return id == 25 ? phone.smart_os == id : phone.smart_os != 25;
      });

      that.showSmartPhonesList(filteredPhones);
    },

    filterPrice: function (price) {
      var that = this;

      const filteredPhones = allSmartPhones.filter((phone) => {
        return price < 50000
          ? phone.smart_current_price >= price - 10000 &&
              phone.smart_current_price <= price
          : phone.smart_current_price >= price - 10000;
      });

      that.showSmartPhonesList(filteredPhones);
    },

    filterType: function (id) {
      var that = this;

      const filteredPhones = allSmartPhones.filter((phone) => {
        return phone.smart_type == id;
      });

      that.showSmartPhonesList(filteredPhones);
    },

    addToCompare: function (id) {
      if (compare.length < maxCompare) {
        const phones_respon = allSmartPhones.filter((phone) => {
          return phone.sma_id == id;
        });
        compare.push(phones_respon[0]);
        this.addCompareItem();
        this.toggleCompare();
      }
    },

    removeFromCompare: function (id) {
      if (compare.length > 0) {
        let index = -1;
        for (let r = 0; r < compare.length; r++) {
          if (compare[r].sma_id == id) {
            index = r;
            break;
          }
        }
        if (index > -1) {
          compare.splice(index, 1);
        }
        this.addCompareItem();
        this.toggleCompare();
      }
    },

    clearCompare: function (id) {
      if (compare.length > 0) {
        compare = [];
        this.addCompareItem();
        this.toggleCompare();
      }
    },

    addCompareItem: function () {
      var container = $("#compare-body");
      const phones_respon = compare;

      container.empty();

      for (let i = 0; i < compare.length; i++) {
        var item = `
            <div class="col-4">
              <div class="row">
                <div class="col-sm-12">
                  <img src="assets/img/files/${phones_respon[i].image_name}" alt="${phones_respon[i].smart_name}" class="img-fluid" />
                </div>
                <div class="col-sm-12">
                  <h4>${phones_respon[i].smart_name}</h4>
                  <h5>Specifications</h5>
                  <hr />
                  <button class="btn btn-danger" onClick="Custom.removeFromCompare(${phones_respon[i].sma_id})"><i class="fas fa-times"></i></button>
                  <div class="row">
                    <div class="col-sm-12">
                      <h6>System Design</h6>
                      <p>OS: <span>${phones_respon[i].os_name}</span></p>
                      <p>CPU: <span>${phones_respon[i].smart_cpu}</span></p>
                      <p>Battery: <span>${phones_respon[i].smart_display_protection}</span></p>
                    </div>
                    <div class="col-sm-12">
                      <h6>Display</h6>
                      <p>Type: <span>${phones_respon[i].smart_display_type}</span></p>
                      <p>Size: <span>${phones_respon[i].smart_display_size}</span></p>
                    </div>
                  </div>
                  <hr />
                  <div class="row">
                    <div class="col-sm-12">
                      <h6>Memory</h6>
                      <p>Size: <span>${phones_respon[i].smart_memory_size}</span></p>
                      <p>Card Slot: <span>${phones_respon[i].smart_card_slot}</span></p>
                    </div>
                    <div class="col-sm-12">
                      <h6>Camera</h6>
                      <p>Front: <span>${phones_respon[i].smart_front_camera}</span></p>
                      <p>Back: <span>${phones_respon[i].smart_back_camera}</span></p>
                    </div>
                  </div>
                  <div class="row">
                    <div class="col-sm-12">
                      <h6>Network</h6>
                      <p>Connection: <span>${phones_respon[i].smart_connection}</span></p>
                      <p>Sim: <span>${phones_respon[i].smart_sim}</span></p>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          `;
        container.append(item);
      }
    },

    toggleCompare: function () {
      if (compare.length > 0) {
        $(".float").html(`<p>Compare ${compare.length} items</p>`);
        $(".float").css("bottom", window.innerHeight / 2);
        $(".float").css("display", "block");
      } else {
        $(".float").css("display", "none");
      }
    },
    
    timerIncrement: function () {
        idleTime = idleTime + 1;
        if (idleTime >= 1) { // 1 minute
            window.location.href = "https://izoneafrica.net/safshop";
        }
    }
  };
})();
